var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
var __defProp2 = Object.defineProperty;
var __defNormalProp2 = (obj, key, value) => key in obj ? __defProp2(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField2 = (obj, key, value) => __defNormalProp2(obj, typeof key !== "symbol" ? key + "" : key, value);
function clean(obj, undefinedOnly = false) {
  if (obj == null) throw new Error("Cannot clean a NULL value");
  if (Array.isArray(obj)) {
    obj = obj.filter((o) => o != null);
  } else {
    Object.entries(obj).forEach(([key, value]) => {
      if (undefinedOnly && value === void 0 || !undefinedOnly && value == null) delete obj[key];
    });
  }
  return obj;
}
function isEqual(a, b) {
  const ta = typeof a, tb = typeof b;
  if (ta != "object" || a == null || (tb != "object" || b == null))
    return ta == "function" && tb == "function" ? a.toString() == b.toString() : a === b;
  const keys = Object.keys(a);
  if (keys.length != Object.keys(b).length) return false;
  return Object.keys(a).every((key) => isEqual(a[key], b[key]));
}
function JSONAttemptParse(json) {
  try {
    return JSON.parse(json);
  } catch {
    return json;
  }
}
function JSONSanitize(obj, space) {
  return JSON.stringify(obj, (key, value) => {
    return value;
  }, space);
}
function makeArray(value) {
  return Array.isArray(value) ? value : [value];
}
class ASet extends Array {
  /** Number of elements in set */
  get size() {
    return this.length;
  }
  /**
   * Array to create set from, duplicate values will be removed
   * @param {T[]} elements Elements which will be added to set
   */
  constructor(elements = []) {
    super();
    if (!!(elements == null ? void 0 : elements["forEach"]))
      elements.forEach((el) => this.add(el));
  }
  /**
   * Add elements to set if unique
   * @param items
   */
  add(...items) {
    items.filter((el) => !this.has(el)).forEach((el) => this.push(el));
    return this;
  }
  /**
   * Delete elements from set
   * @param items Elements that will be deleted
   */
  delete(...items) {
    items.forEach((el) => {
      const index = this.indexOf(el);
      if (index != -1) this.splice(index, 1);
    });
    return this;
  }
  /**
   * Create list of elements this set has which the comparison set does not
   * @param {ASet<T>} set Set to compare against
   * @return {ASet<T>} Different elements
   */
  difference(set) {
    return new ASet(this.filter((el) => !set.has(el)));
  }
  /**
   * Check if set includes element
   * @param {T} el Element to look for
   * @return {boolean} True if element was found, false otherwise
   */
  has(el) {
    return this.indexOf(el) != -1;
  }
  /**
   * Find index number of element, or -1 if it doesn't exist. Matches by equality not reference
   *
   * @param {T} search Element to find
   * @param {number} fromIndex Starting index position
   * @return {number} Element index number or -1 if missing
   */
  indexOf(search, fromIndex) {
    return super.findIndex((el) => isEqual(el, search), fromIndex);
  }
  /**
   * Create list of elements this set has in common with the comparison set
   * @param {ASet<T>} set Set to compare against
   * @return {boolean} Set of common elements
   */
  intersection(set) {
    return new ASet(this.filter((el) => set.has(el)));
  }
  /**
   * Check if this set has no elements in common with the comparison set
   * @param {ASet<T>} set Set to compare against
   * @return {boolean} True if nothing in common, false otherwise
   */
  isDisjointFrom(set) {
    return this.intersection(set).size == 0;
  }
  /**
   * Check if all elements in this set are included in the comparison set
   * @param {ASet<T>} set Set to compare against
   * @return {boolean} True if all elements are included, false otherwise
   */
  isSubsetOf(set) {
    return this.findIndex((el) => !set.has(el)) == -1;
  }
  /**
   * Check if all elements from comparison set are included in this set
   * @param {ASet<T>} set Set to compare against
   * @return {boolean} True if all elements are included, false otherwise
   */
  isSuperset(set) {
    return set.findIndex((el) => !this.has(el)) == -1;
  }
  /**
   * Create list of elements that are only in one set but not both (XOR)
   * @param {ASet<T>} set Set to compare against
   * @return {ASet<T>} New set of unique elements
   */
  symmetricDifference(set) {
    return new ASet([...this.difference(set), ...set.difference(this)]);
  }
  /**
   * Create joined list of elements included in this & the comparison set
   * @param {ASet<T>} set Set join
   * @return {ASet<T>} New set of both previous sets combined
   */
  union(set) {
    return new ASet([...this, ...set]);
  }
}
class Cache {
  /**
   * Create new cache
   *
   * @param {keyof T} key Default property to use as primary key
   * @param options
   */
  constructor(key, options = {}) {
    __publicField2(this, "store", {});
    __publicField2(this, "complete", false);
    __publicField2(this, "values", this.all());
    this.key = key;
    this.options = options;
    if (options.storageKey && !options.storage && typeof Storage !== "undefined")
      options.storage = localStorage;
    if (options.storageKey && options.storage) {
      const stored = options.storage.getItem(options.storageKey);
      if (stored) {
        try {
          Object.assign(this.store, JSON.parse(stored));
        } catch {
        }
      }
    }
    return new Proxy(this, {
      get: (target, prop) => {
        if (prop in target) return target[prop];
        return target.store[prop];
      },
      set: (target, prop, value) => {
        if (prop in target) target[prop] = value;
        else target.store[prop] = value;
        return true;
      }
    });
  }
  getKey(value) {
    if (!this.key) throw new Error("No key defined");
    return value[this.key];
  }
  /**
   * Get all cached items
   *
   * @return {T[]} Array of items
   */
  all() {
    return Object.values(this.store);
  }
  /**
   * Add a new item to the cache. Like set, but finds key automatically
   *
   * @param {T} value Item to add to cache
   * @param {number | undefined} ttl Override default expiry
   * @return {this}
   */
  add(value, ttl = this.ttl) {
    const key = this.getKey(value);
    this.set(key, value, ttl);
    return this;
  }
  /**
   * Add several rows to the cache
   *
   * @param {T[]} rows Several items that will be cached using the default key
   * @param complete Mark cache as complete & reliable, defaults to true
   * @return {this}
   */
  addAll(rows, complete = true) {
    rows.forEach((r) => this.add(r));
    this.complete = complete;
    return this;
  }
  /**
   * Remove all keys from cache
   */
  clear() {
    this.store = {};
  }
  /**
   * Delete an item from the cache
   *
   * @param {K} key Item's primary key
   */
  delete(key) {
    delete this.store[key];
    if (this.options.storageKey && this.options.storage)
      this.options.storage.setItem(this.options.storageKey, JSON.stringify(this.store));
  }
  /**
   * Return cache as an array of key-value pairs
   * @return {[K, T][]} Key-value pairs array
   */
  entries() {
    return Object.entries(this.store);
  }
  /**
   * Get item from the cache
   * @param {K} key Key to lookup
   * @return {T} Cached item
   */
  get(key) {
    return this.store[key];
  }
  /**
   * Get a list of cached keys
   *
   * @return {K[]} Array of keys
   */
  keys() {
    return Object.keys(this.store);
  }
  /**
   * Get map of cached items
   *
   * @return {Record<K, T>}
   */
  map() {
    return structuredClone(this.store);
  }
  /**
   * Add an item to the cache manually specifying the key
   *
   * @param {K} key Key item will be cached under
   * @param {T} value Item to cache
   * @param {number | undefined} ttl Override default expiry in seconds
   * @return {this}
   */
  set(key, value, ttl = this.options.ttl) {
    this.store[key] = value;
    if (this.options.storageKey && this.options.storage)
      this.options.storage.setItem(this.options.storageKey, JSON.stringify(this.store));
    if (ttl) setTimeout(() => {
      this.complete = false;
      this.delete(key);
    }, ttl * 1e3);
    return this;
  }
}
class PromiseProgress extends Promise {
  constructor(executor) {
    super((resolve, reject) => executor(
      (value) => resolve(value),
      (reason) => reject(reason),
      (progress) => this.progress = progress
    ));
    __publicField2(this, "listeners", []);
    __publicField2(this, "_progress", 0);
  }
  get progress() {
    return this._progress;
  }
  set progress(p) {
    if (p == this._progress) return;
    this._progress = p;
    this.listeners.forEach((l) => l(p));
  }
  static from(promise) {
    if (promise instanceof PromiseProgress) return promise;
    return new PromiseProgress((res, rej) => promise.then((...args) => res(...args)).catch((...args) => rej(...args)));
  }
  from(promise) {
    const newPromise = PromiseProgress.from(promise);
    this.onProgress((p) => newPromise.progress = p);
    return newPromise;
  }
  onProgress(callback) {
    this.listeners.push(callback);
    return this;
  }
  then(res, rej) {
    const resp = super.then(res, rej);
    return this.from(resp);
  }
  catch(rej) {
    return this.from(super.catch(rej));
  }
  finally(res) {
    return this.from(super.finally(res));
  }
}
function downloadFile(blob, name) {
  if (!(blob instanceof Blob)) blob = new Blob(makeArray(blob));
  const url = URL.createObjectURL(blob);
  downloadUrl(url, name);
  URL.revokeObjectURL(url);
}
function downloadUrl(href, name) {
  const a = document.createElement("a");
  a.href = href;
  a.download = name || href.split("/").pop();
  document.body.appendChild(a);
  a.click();
  document.body.removeChild(a);
}
function fileBrowser(options = {}) {
  return new Promise((res) => {
    const input = document.createElement("input");
    input.type = "file";
    input.accept = options.accept || "*";
    input.style.display = "none";
    input.multiple = !!options.multiple;
    input.onblur = input.onchange = async () => {
      res(Array.from(input.files));
      input.remove();
    };
    document.body.appendChild(input);
    input.click();
  });
}
function timestampFilename(name, date = /* @__PURE__ */ new Date()) {
  if (typeof date == "number" || typeof date == "string") date = new Date(date);
  const timestamp = `${date.getFullYear()}-${(date.getMonth() + 1).toString().padStart(2, "0")}-${date.getDate().toString().padStart(2, "0")}_${date.getHours().toString().padStart(2, "0")}-${date.getMinutes().toString().padStart(2, "0")}-${date.getSeconds().toString().padStart(2, "0")}`;
  return timestamp;
}
function uploadWithProgress(options) {
  return new PromiseProgress((res, rej, prog) => {
    const xhr = new XMLHttpRequest();
    const formData2 = new FormData();
    options.files.forEach((f) => formData2.append("file", f));
    xhr.withCredentials = !!options.withCredentials;
    xhr.upload.addEventListener("progress", (event) => event.lengthComputable ? prog(event.loaded / event.total) : null);
    xhr.addEventListener("loadend", () => res(JSONAttemptParse(xhr.responseText)));
    xhr.addEventListener("error", () => rej(JSONAttemptParse(xhr.responseText)));
    xhr.addEventListener("timeout", () => rej({ error: "Request timed out" }));
    xhr.open("POST", options.url);
    Object.entries(options.headers || {}).forEach(([key, value]) => xhr.setRequestHeader(key, value));
    xhr.send(formData2);
  });
}
class TypedEmitter {
  constructor() {
    __publicField2(this, "listeners", {});
  }
  static emit(event, ...args) {
    (this.listeners["*"] || []).forEach((l) => l(event, ...args));
    (this.listeners[event.toString()] || []).forEach((l) => l(...args));
  }
  static off(event, listener) {
    const e = event.toString();
    this.listeners[e] = (this.listeners[e] || []).filter((l) => l === listener);
  }
  static on(event, listener) {
    var _a;
    const e = event.toString();
    if (!this.listeners[e]) this.listeners[e] = [];
    (_a = this.listeners[e]) == null ? void 0 : _a.push(listener);
    return () => this.off(event, listener);
  }
  static once(event, listener) {
    return new Promise((res) => {
      const unsubscribe = this.on(event, (...args) => {
        res(args.length == 1 ? args[0] : args);
        if (listener) listener(...args);
        unsubscribe();
      });
    });
  }
  emit(event, ...args) {
    (this.listeners["*"] || []).forEach((l) => l(event, ...args));
    (this.listeners[event] || []).forEach((l) => l(...args));
  }
  off(event, listener) {
    this.listeners[event] = (this.listeners[event] || []).filter((l) => l === listener);
  }
  on(event, listener) {
    var _a;
    if (!this.listeners[event]) this.listeners[event] = [];
    (_a = this.listeners[event]) == null ? void 0 : _a.push(listener);
    return () => this.off(event, listener);
  }
  once(event, listener) {
    return new Promise((res) => {
      const unsubscribe = this.on(event, (...args) => {
        res(args.length == 1 ? args[0] : args);
        if (listener) listener(...args);
        unsubscribe();
      });
    });
  }
}
__publicField2(TypedEmitter, "listeners", {});
class CustomError extends Error {
  constructor(message, code) {
    super(message);
    __publicField2(this, "_code");
    if (code != null) this._code = code;
  }
  get code() {
    return this._code || this.constructor.code;
  }
  set code(c) {
    this._code = c;
  }
  static from(err) {
    const code = Number(err.statusCode) ?? Number(err.code);
    const newErr = new this(err.message || err.toString());
    return Object.assign(newErr, {
      stack: err.stack,
      ...err,
      code: code ?? void 0
    });
  }
  static instanceof(err) {
    return err.constructor.code != void 0;
  }
  toString() {
    return this.message || super.toString();
  }
}
__publicField2(CustomError, "code", 500);
class BadRequestError extends CustomError {
  constructor(message = "Bad Request") {
    super(message);
  }
  static instanceof(err) {
    return err.constructor.code == this.code;
  }
}
__publicField2(BadRequestError, "code", 400);
class UnauthorizedError extends CustomError {
  constructor(message = "Unauthorized") {
    super(message);
  }
  static instanceof(err) {
    return err.constructor.code == this.code;
  }
}
__publicField2(UnauthorizedError, "code", 401);
class PaymentRequiredError extends CustomError {
  constructor(message = "Payment Required") {
    super(message);
  }
  static instanceof(err) {
    return err.constructor.code == this.code;
  }
}
__publicField2(PaymentRequiredError, "code", 402);
class ForbiddenError extends CustomError {
  constructor(message = "Forbidden") {
    super(message);
  }
  static instanceof(err) {
    return err.constructor.code == this.code;
  }
}
__publicField2(ForbiddenError, "code", 403);
class NotFoundError extends CustomError {
  constructor(message = "Not Found") {
    super(message);
  }
  static instanceof(err) {
    return err.constructor.code == this.code;
  }
}
__publicField2(NotFoundError, "code", 404);
class MethodNotAllowedError extends CustomError {
  constructor(message = "Method Not Allowed") {
    super(message);
  }
  static instanceof(err) {
    return err.constructor.code == this.code;
  }
}
__publicField2(MethodNotAllowedError, "code", 405);
class NotAcceptableError extends CustomError {
  constructor(message = "Not Acceptable") {
    super(message);
  }
  static instanceof(err) {
    return err.constructor.code == this.code;
  }
}
__publicField2(NotAcceptableError, "code", 406);
class InternalServerError extends CustomError {
  constructor(message = "Internal Server Error") {
    super(message);
  }
  static instanceof(err) {
    return err.constructor.code == this.code;
  }
}
__publicField2(InternalServerError, "code", 500);
class NotImplementedError extends CustomError {
  constructor(message = "Not Implemented") {
    super(message);
  }
  static instanceof(err) {
    return err.constructor.code == this.code;
  }
}
__publicField2(NotImplementedError, "code", 501);
class BadGatewayError extends CustomError {
  constructor(message = "Bad Gateway") {
    super(message);
  }
  static instanceof(err) {
    return err.constructor.code == this.code;
  }
}
__publicField2(BadGatewayError, "code", 502);
class ServiceUnavailableError extends CustomError {
  constructor(message = "Service Unavailable") {
    super(message);
  }
  static instanceof(err) {
    return err.constructor.code == this.code;
  }
}
__publicField2(ServiceUnavailableError, "code", 503);
class GatewayTimeoutError extends CustomError {
  constructor(message = "Gateway Timeout") {
    super(message);
  }
  static instanceof(err) {
    return err.constructor.code == this.code;
  }
}
__publicField2(GatewayTimeoutError, "code", 504);
const _Http = class _Http2 {
  constructor(defaults = {}) {
    __publicField2(this, "interceptors", {});
    __publicField2(this, "headers", {});
    __publicField2(this, "url");
    this.url = defaults.url ?? null;
    this.headers = defaults.headers || {};
    if (defaults.interceptors) {
      defaults.interceptors.forEach((i) => _Http2.addInterceptor(i));
    }
  }
  static addInterceptor(fn) {
    const key = Object.keys(_Http2.interceptors).length.toString();
    _Http2.interceptors[key] = fn;
    return () => {
      _Http2.interceptors[key] = null;
    };
  }
  addInterceptor(fn) {
    const key = Object.keys(this.interceptors).length.toString();
    this.interceptors[key] = fn;
    return () => {
      this.interceptors[key] = null;
    };
  }
  request(opts = {}) {
    var _a;
    if (!this.url && !opts.url) throw new Error("URL needs to be set");
    let url = (((_a = opts.url) == null ? void 0 : _a.startsWith("http")) ? opts.url : (this.url || "") + (opts.url || "")).replace(/([^:]\/)\/+/g, "$1");
    if (opts.fragment) url.includes("#") ? url.replace(/#.*(\?|\n)/g, (match, arg1) => `#${opts.fragment}${arg1}`) : url += "#" + opts.fragment;
    if (opts.query) {
      const q = Array.isArray(opts.query) ? opts.query : Object.keys(opts.query).map((k) => ({ key: k, value: opts.query[k] }));
      url += (url.includes("?") ? "&" : "?") + q.map((q2) => `${q2.key}=${q2.value}`).join("&");
    }
    const headers = clean({
      "Content-Type": !opts.body ? void 0 : opts.body instanceof FormData ? "multipart/form-data" : "application/json",
      ..._Http2.headers,
      ...this.headers,
      ...opts.headers
    });
    if (typeof opts.body == "object" && opts.body != null && headers["Content-Type"] == "application/json")
      opts.body = JSON.stringify(opts.body);
    return new PromiseProgress((res, rej, prog) => {
      try {
        fetch(url, {
          headers,
          method: opts.method || (opts.body ? "POST" : "GET"),
          body: opts.body
        }).then(async (resp) => {
          var _a2, _b;
          for (let fn of [...Object.values(_Http2.interceptors), ...Object.values(this.interceptors)]) {
            await new Promise((res2) => fn(resp, () => res2()));
          }
          const contentLength = resp.headers.get("Content-Length");
          const total = contentLength ? parseInt(contentLength, 10) : 0;
          let loaded = 0;
          const reader = (_a2 = resp.body) == null ? void 0 : _a2.getReader();
          const stream = new ReadableStream({
            start(controller) {
              function push() {
                reader == null ? void 0 : reader.read().then((event) => {
                  if (event.done) return controller.close();
                  loaded += event.value.byteLength;
                  prog(loaded / total);
                  controller.enqueue(event.value);
                  push();
                }).catch((error) => controller.error(error));
              }
              push();
            }
          });
          resp.data = new Response(stream);
          if (opts.decode == null || opts.decode) {
            const content = (_b = resp.headers.get("Content-Type")) == null ? void 0 : _b.toLowerCase();
            if (content == null ? void 0 : content.includes("form")) resp.data = await resp.data.formData();
            else if (content == null ? void 0 : content.includes("json")) resp.data = await resp.data.json();
            else if (content == null ? void 0 : content.includes("text")) resp.data = await resp.data.text();
            else if (content == null ? void 0 : content.includes("application")) resp.data = await resp.data.blob();
          }
          if (resp.ok) res(resp);
          else rej(resp);
        }).catch((err) => rej(err));
      } catch (err) {
        rej(err);
      }
    });
  }
};
__publicField2(_Http, "interceptors", {});
__publicField2(_Http, "headers", {});
let Http = _Http;
const CliEffects = {
  CLEAR: "\x1B[0m",
  BRIGHT: "\x1B[1m",
  DIM: "\x1B[2m",
  UNDERSCORE: "\x1B[4m",
  BLINK: "\x1B[5m",
  REVERSE: "\x1B[7m",
  HIDDEN: "\x1B[8m"
};
const CliForeground = {
  BLACK: "\x1B[30m",
  RED: "\x1B[31m",
  GREEN: "\x1B[32m",
  YELLOW: "\x1B[33m",
  BLUE: "\x1B[34m",
  MAGENTA: "\x1B[35m",
  CYAN: "\x1B[36m",
  LIGHT_GREY: "\x1B[37m",
  GREY: "\x1B[90m",
  LIGHT_RED: "\x1B[91m",
  LIGHT_GREEN: "\x1B[92m",
  LIGHT_YELLOW: "\x1B[93m",
  LIGHT_BLUE: "\x1B[94m",
  LIGHT_MAGENTA: "\x1B[95m",
  LIGHT_CYAN: "\x1B[96m",
  WHITE: "\x1B[97m"
};
var LOG_LEVEL = /* @__PURE__ */ ((LOG_LEVEL2) => {
  LOG_LEVEL2[LOG_LEVEL2["ERROR"] = 0] = "ERROR";
  LOG_LEVEL2[LOG_LEVEL2["WARN"] = 1] = "WARN";
  LOG_LEVEL2[LOG_LEVEL2["INFO"] = 2] = "INFO";
  LOG_LEVEL2[LOG_LEVEL2["LOG"] = 3] = "LOG";
  LOG_LEVEL2[LOG_LEVEL2["DEBUG"] = 4] = "DEBUG";
  return LOG_LEVEL2;
})(LOG_LEVEL || {});
const _Logger = class _Logger2 extends TypedEmitter {
  constructor(namespace) {
    super();
    this.namespace = namespace;
  }
  format(...text) {
    const now = /* @__PURE__ */ new Date();
    const timestamp = `${now.getFullYear()}-${now.getMonth() + 1}-${now.getDate()} ${now.getHours().toString().padStart(2, "0")}:${now.getMinutes().toString().padStart(2, "0")}:${now.getSeconds().toString().padStart(2, "0")}.${now.getMilliseconds().toString().padEnd(3, "0")}`;
    return `${timestamp}${this.namespace ? ` [${this.namespace}]` : ""} ${text.map((t) => typeof t == "string" ? t : JSONSanitize(t, 2)).join(" ")}`;
  }
  debug(...args) {
    if (_Logger2.LOG_LEVEL < 4) return;
    const str = this.format(...args);
    _Logger2.emit(4, str);
    console.debug(CliForeground.LIGHT_GREY + str + CliEffects.CLEAR);
  }
  log(...args) {
    if (_Logger2.LOG_LEVEL < 3) return;
    const str = this.format(...args);
    _Logger2.emit(3, str);
    console.log(CliEffects.CLEAR + str);
  }
  info(...args) {
    if (_Logger2.LOG_LEVEL < 2) return;
    const str = this.format(...args);
    _Logger2.emit(2, str);
    console.info(CliForeground.BLUE + str + CliEffects.CLEAR);
  }
  warn(...args) {
    if (_Logger2.LOG_LEVEL < 1) return;
    const str = this.format(...args);
    _Logger2.emit(1, str);
    console.warn(CliForeground.YELLOW + str + CliEffects.CLEAR);
  }
  error(...args) {
    if (_Logger2.LOG_LEVEL < 0) return;
    const str = this.format(...args);
    _Logger2.emit(0, str);
    console.error(CliForeground.RED + str + CliEffects.CLEAR);
  }
};
__publicField2(_Logger, "LOG_LEVEL", 4);
function PE(str, ...args) {
  const combined = [];
  for (let i = 0; i < str.length || i < args.length; i++) {
    if (str[i]) combined.push(str[i]);
    if (args[i]) combined.push(args[i]);
  }
  return new PathEvent(combined.join(""));
}
function PES(str, ...args) {
  let combined = [];
  for (let i = 0; i < str.length || i < args.length; i++) {
    if (str[i]) combined.push(str[i]);
    if (args[i]) combined.push(args[i]);
  }
  const [paths, methods] = combined.join("").split(":");
  return PathEvent.toString(paths, methods == null ? void 0 : methods.split(""));
}
class PathError extends Error {
}
class PathEvent {
  constructor(Event) {
    __publicField2(this, "module");
    __publicField2(this, "fullPath");
    __publicField2(this, "path");
    __publicField2(this, "name");
    __publicField2(this, "methods");
    var _a;
    if (typeof Event == "object") return Object.assign(this, Event);
    let [p, scope, method] = Event.replaceAll(/\/{2,}/g, "/").split(":");
    if (!method) method = scope || "*";
    if (p == "*" || !p && method == "*") {
      p = "";
      method = "*";
    }
    let temp = p.split("/").filter((p2) => !!p2);
    this.module = ((_a = temp.splice(0, 1)[0]) == null ? void 0 : _a.toLowerCase()) || "";
    this.fullPath = p;
    this.path = temp.join("/");
    this.name = temp.pop() || "";
    this.methods = new ASet(method.split(""));
  }
  /** All/Wildcard specified */
  get all() {
    return this.methods.has("*");
  }
  set all(v) {
    v ? new ASet(["*"]) : this.methods.delete("*");
  }
  /** None specified */
  get none() {
    return this.methods.has("n");
  }
  set none(v) {
    v ? this.methods = new ASet(["n"]) : this.methods.delete("n");
  }
  /** Create method specified */
  get create() {
    return !this.methods.has("n") && (this.methods.has("*") || this.methods.has("c"));
  }
  set create(v) {
    v ? this.methods.delete("n").add("c") : this.methods.delete("c");
  }
  /** Read method specified */
  get read() {
    return !this.methods.has("n") && (this.methods.has("*") || this.methods.has("r"));
  }
  set read(v) {
    v ? this.methods.delete("n").add("r") : this.methods.delete("r");
  }
  /** Update method specified */
  get update() {
    return !this.methods.has("n") && (this.methods.has("*") || this.methods.has("u"));
  }
  set update(v) {
    v ? this.methods.delete("n").add("u") : this.methods.delete("u");
  }
  /** Delete method specified */
  get delete() {
    return !this.methods.has("n") && (this.methods.has("*") || this.methods.has("d"));
  }
  set delete(v) {
    v ? this.methods.delete("n").add("d") : this.methods.delete("d");
  }
  /**
   * Combine multiple  events into one parsed object. Longest path takes precedent, but all subsequent methods are
   * combined until a "none" is reached
   *
   * @param {string | PathEvent} paths Events as strings or pre-parsed
   * @return {PathEvent} Final combined permission
   */
  static combine(...paths) {
    let hitNone = false;
    const combined = paths.map((p) => new PathEvent(p)).toSorted((p1, p2) => {
      const l1 = p1.fullPath.length, l2 = p2.fullPath.length;
      return l1 < l2 ? 1 : l1 > l2 ? -1 : 0;
    }).reduce((acc, p) => {
      if (p.none) hitNone = true;
      if (!acc) return p;
      if (hitNone) return acc;
      acc.methods = [...acc.methods, ...p.methods];
      return acc;
    }, null);
    combined.methods = new ASet(combined.methods);
    return combined;
  }
  /**
   * Filter a set of paths based on the target
   *
   * @param {string | PathEvent | (string | PathEvent)[]} target Array of events that will filtered
   * @param filter {...PathEvent} Must container one of
   * @return {boolean} Whether there is any overlap
   */
  static filter(target, ...filter) {
    const parsedTarget = makeArray(target).map((pe) => new PathEvent(pe));
    const parsedFind = makeArray(filter).map((pe) => new PathEvent(pe));
    return parsedTarget.filter((t) => {
      if (!t.fullPath && t.all) return true;
      return !!parsedFind.find((f) => t.fullPath.startsWith(f.fullPath) && (f.all || t.all || t.methods.intersection(f.methods).length));
    });
  }
  /**
   * Squash 2 sets of paths & return true if any overlap is found
   *
   * @param {string | PathEvent | (string | PathEvent)[]} target Array of Events as strings or pre-parsed
   * @param has Target must have at least one of these path
   * @return {boolean} Whether there is any overlap
   */
  static has(target, ...has) {
    const parsedRequired = makeArray(has).map((pe) => new PathEvent(pe));
    const parsedTarget = makeArray(target).map((pe) => new PathEvent(pe));
    return !!parsedRequired.find((r) => {
      if (!r.fullPath && r.all) return true;
      const filtered = parsedTarget.filter((p) => r.fullPath.startsWith(p.fullPath));
      if (!filtered.length) return false;
      const combined = PathEvent.combine(...filtered);
      return !combined.none && (combined.all || r.all) || combined.methods.intersection(r.methods).length;
    });
  }
  /**
   * Squash 2 sets of paths & return true if the target has all paths
   *
   * @param {string | PathEvent | (string | PathEvent)[]} target Array of Events as strings or pre-parsed
   * @param has Target must have all these paths
   * @return {boolean} Whether there is any overlap
   */
  static hasAll(target, ...has) {
    return has.filter((h) => PathEvent.has(target, h)).length == has.length;
  }
  /**
   * Same as `has` but raises an error if there is no overlap
   *
   * @param {string | string[]} target Array of Events as strings or pre-parsed
   * @param has Target must have at least one of these path
   */
  static hasFatal(target, ...has) {
    if (!PathEvent.has(target, ...has)) throw new PathError(`Requires one of: ${makeArray(has).join(", ")}`);
  }
  /**
   * Same as `hasAll` but raises an error if the target is missing any paths
   *
   * @param {string | string[]} target Array of Events as strings or pre-parsed
   * @param has Target must have all these paths
   */
  static hasAllFatal(target, ...has) {
    if (!PathEvent.hasAll(target, ...has)) throw new PathError(`Requires all: ${makeArray(has).join(", ")}`);
  }
  /**
   * Create  event string from its components
   *
   * @param {string | string[]} path Event path
   * @param {Method} methods Event method
   * @return {string} String representation of Event
   */
  static toString(path, methods) {
    let p = makeArray(path).filter((p2) => p2 != null).join("/");
    p = p == null ? void 0 : p.trim().replaceAll(/\/{2,}/g, "/").replaceAll(/(^\/|\/$)/g, "");
    if (methods == null ? void 0 : methods.length) p += `:${makeArray(methods).map((m) => m.toLowerCase()).join("")}`;
    return p;
  }
  /**
   * Filter a set of paths based on this event
   *
   * @param {string | PathEvent | (string | PathEvent)[]} target Array of events that will filtered
   * @return {boolean} Whether there is any overlap
   */
  filter(target) {
    return PathEvent.filter(target, this);
  }
  /**
   * Create  event string from its components
   *
   * @return {string} String representation of Event
   */
  toString() {
    return PathEvent.toString(this.fullPath, this.methods);
  }
}
class PathEventEmitter {
  constructor() {
    __publicField2(this, "listeners", []);
  }
  emit(event, ...args) {
    const parsed = new PathEvent(event);
    this.listeners.filter((l) => PathEvent.has(l[0], event)).forEach(async (l) => l[1](parsed, ...args));
  }
  off(listener) {
    this.listeners = this.listeners.filter((l) => l[1] != listener);
  }
  on(event, listener) {
    makeArray(event).forEach((e) => this.listeners.push([new PathEvent(e), listener]));
    return () => this.off(listener);
  }
  once(event, listener) {
    return new Promise((res) => {
      const unsubscribe = this.on(event, (event2, ...args) => {
        res(args.length < 2 ? args[0] : args);
        if (listener) listener(event2, ...args);
        unsubscribe();
      });
    });
  }
  relayEvents(emitter) {
    emitter.on("*", (event, ...args) => this.emit(event, ...args));
  }
}
class Api extends Http {
  constructor(url = location.origin, opts = {}) {
    super({ ...(opts == null ? void 0 : opts.http) || {}, url });
    __publicField(this, "emitter", new PathEventEmitter());
    __publicField(this, "pending", {});
    __publicField(this, "storageKey");
    __publicField(this, "host");
    __publicField(this, "_token", null);
    __publicField(this, "emit", this.emitter.emit.bind(this.emitter));
    __publicField(this, "off", this.emitter.off.bind(this.emitter));
    __publicField(this, "on", this.emitter.on.bind(this.emitter));
    __publicField(this, "once", this.emitter.once.bind(this.emitter));
    __publicField(this, "relayEvents", this.emitter.relayEvents.bind(this.emitter));
    this.url = url;
    this.opts = opts;
    this.host = new URL(url).host;
    this.storageKey = `momentum:token:${this.host}`;
    if (opts.persist && localStorage) {
      const token = localStorage.getItem(this.storageKey);
      if (token) this.token = token;
    }
  }
  get token() {
    return this._token;
  }
  set token(token) {
    if (token == this._token) return;
    this._token = token;
    this.headers["Authorization"] = token ? `Bearer ${token}` : null;
    if (this.opts.persist && localStorage) {
      if (token) localStorage.setItem(this.storageKey, token);
      else localStorage.removeItem(this.storageKey);
    }
    this.emit(PES`api/token:${token ? "u" : "d"}`, token);
  }
  healthcheck() {
    return this.request({ url: "/api/healthcheck" }).then((resp) => {
      this.emit(PES`api/healthcheck:r`, resp);
      return resp;
    });
  }
  request(options) {
    const key = JSONSanitize(options);
    const method = options.method == "GET" ? "r" : options.method == "POST" ? "c" : options.method == "DELETE" ? "d" : "u";
    if (this.pending[key] != null) return this.pending[key];
    this.pending[key] = super.request(options).then((response) => {
      this.emit(PES`api/response:${method}`, { request: options, response });
      return response.data;
    }).catch((err) => {
      const e = (err == null ? void 0 : err.data) || err;
      this.emit(PES`api/error:${method}`, { request: options, error: e });
      throw e;
    }).finally(() => delete this.pending[key]);
    this.emit(PES`api/request:${method}`, { request: options, response: this.pending[key] });
    return this.pending[key];
  }
}
var ActionType = /* @__PURE__ */ ((ActionType2) => {
  ActionType2[ActionType2["CRON"] = 0] = "CRON";
  ActionType2[ActionType2["EVENT"] = 1] = "EVENT";
  ActionType2[ActionType2["DELETE"] = 2] = "DELETE";
  ActionType2[ActionType2["GET"] = 3] = "GET";
  ActionType2[ActionType2["PATCH"] = 4] = "PATCH";
  ActionType2[ActionType2["POST"] = 5] = "POST";
  ActionType2[ActionType2["PUT"] = 6] = "PUT";
  return ActionType2;
})(ActionType || {});
class Actions extends PathEventEmitter {
  constructor(api) {
    super();
    __publicField(this, "api");
    __publicField(this, "cache", new Cache("_id"));
    this.api = typeof api == "string" ? new Api(api) : api;
  }
  async all(reload) {
    if (!reload && this.cache.complete) return this.cache.all();
    return this.api.request({ url: `/api/` + PES`actions` }).then((resp) => {
      this.cache.addAll(resp);
      this.emit(PES`actions:r`, resp || []);
      return resp;
    });
  }
  delete(id) {
    if (!id) throw new Error("Cannot delete action, missing ID");
    return this.api.request({ url: `/api/` + PES`actions/${id}`, method: "DELETE" }).then(() => {
      this.cache.delete(id);
      this.emit(PES`actions/${id}:d`, id);
    });
  }
  read(id, reload = false) {
    if (!id) throw new Error("Cannot read action, missing ID");
    const cached = this.cache.get(id);
    if (!reload && cached) return Promise.resolve(cached);
    return this.api.request({ url: `/api/` + PES`actions/${id}` }).then((action) => {
      if (action) this.cache.add(action);
      this.emit(PES`actions/${id}:r`, action);
      return action;
    });
  }
  run(path, opts = {}) {
    if (!path) throw new Error("Cannot run action, missing path");
    return this.api.request({ url: `/api/` + PES`actions/run/${path}`, ...opts });
  }
  runById(action, opts = {}) {
    const id = typeof action == "string" ? action : action == null ? void 0 : action._id;
    if (!id) throw new Error("Cannot run action, missing ID");
    return this.api.request({ url: "/api/" + PES`actions/run-by-id/${id}`, method: "POST", ...opts });
  }
  update(action) {
    return this.api.request({
      url: `/api/` + PES`actions/${action._id}`,
      method: "POST",
      body: action
    }).then((action2) => {
      if (action2) this.cache.add(action2);
      this.emit(PES`actions/${action2._id}:u`, action2);
      return action2;
    });
  }
}
class Ai extends PathEventEmitter {
  constructor(api) {
    super();
    __publicField(this, "api");
    this.api = typeof api == "string" ? new Api(api) : api;
  }
  ask(question, context) {
    if (!question) throw new Error("Cannot ask AI, missing question");
    return this.api.request({ url: `/api/` + PES`ai`, method: "POST", body: {
      question,
      context
    } }).then((response) => {
      this.emit(PES`ai:c`, { question, context, response });
      return response;
    });
  }
  clear() {
    return this.api.request({ url: "/api/" + PES`ai`, method: "DELETE" }).then(() => this.emit(PES`ai:d`, this.api.token));
  }
}
class Analytics extends PathEventEmitter {
  constructor(api) {
    super();
    __publicField(this, "api");
    this.api = typeof api == "string" ? new Api(api) : api;
  }
  ipTrace(ip) {
    if (!ip) throw new Error("Cannot trace, missing IP");
    return this.api.request({ url: `/api/` + PES`analytics/trace/${ip}` }).then((resp) => {
      this.emit(PES`analytics/trace/${ip}:r`, resp);
      return resp;
    });
  }
}
class Auth extends PathEventEmitter {
  constructor(api, opts = {}) {
    super();
    __publicField(this, "api");
    __publicField(this, "_permissions", []);
    __publicField(this, "_user");
    // Permission helpers
    __publicField(this, "filter", (...events) => PathEvent.filter(this.permissions, ...events));
    __publicField(this, "has", (...events) => PathEvent.has(this.permissions, ...events));
    __publicField(this, "hasAll", (...events) => PathEvent.hasAll(this.permissions, ...events));
    __publicField(this, "hasFatal", (...events) => PathEvent.hasFatal(this.permissions, ...events));
    __publicField(this, "hasAllFatal", (...events) => PathEvent.hasAllFatal(this.permissions, ...events));
    __publicField(this, "enableTotp", this.resetTotp);
    this.opts = opts;
    this.api = typeof api == "string" ? new Api(api) : api;
    this.opts = {
      loginUrl: this.api.url + "/ui/#/login",
      ...this.opts
    };
    this.api.addInterceptor((resp, next) => {
      const blacklist = [
        "/api/auth/login",
        "/api/auth/password",
        "/api/auth/totp"
      ];
      if (resp.status == 401 && !blacklist.find((url) => resp.url.includes(url)))
        this.emit(PES`auth/session-expired:d`, this.api.token);
      next();
    });
    if (this.api.token) this.session(this.api.token, true);
    else this.user = null;
    this.api.on("api/token", (event, token) => {
      if (token) this.session(token, true).catch(() => {
      });
      else this.user = null;
    });
  }
  get permissions() {
    return this._permissions;
  }
  set permissions(perms) {
    this._permissions = perms;
    this.emit(PES`auth/permissions:u`, this._permissions);
  }
  get user() {
    return this._user;
  }
  set user(user) {
    if (!isEqual(this.user, user)) {
      this._user = user ? user : null;
      this.emit(PES`auth/user:u`, this._user);
    }
  }
  knownHost(host = location.origin) {
    if (host.startsWith("/")) return Promise.resolve();
    return this.api.request({ url: `/api/auth/known-host?host=${encodeURI(new URL(host).origin)}` });
  }
  login(username, password, totp) {
    if (!username || !password) throw new Error("Cannot login, missing username or password");
    return this.api.request({
      url: "/api/auth/login",
      headers: { Authorization: void 0 },
      method: "POST",
      body: {
        username: username.trim(),
        password: password.trim(),
        totp
      }
    }).then(async (resp) => {
      this.api.token = (resp == null ? void 0 : resp.token) || null;
      const user = await this.once("auth/user");
      this.emit(PES`auth/login/${username}:u`, user);
      return user;
    });
  }
  loginRedirect(host = location.origin) {
    return new Promise((res, rej) => {
      var _a;
      const win = window.open(encodeURI(`${(_a = this.opts) == null ? void 0 : _a.loginUrl}?redirect=postmessage&host=${host}`), "_blank");
      if (!win) return rej("Unable to open login");
      const origin = new URL(this.opts.loginUrl).origin;
      win.addEventListener("message", (event) => {
        const data = (event == null ? void 0 : event.data) || {};
        if (event.origin != data.sender || data.sender != origin) return;
        if (!data.token) return rej("Unknown response from login");
        this.api.token = event.data.token;
        res(event.data.token);
        win.close();
      });
    });
  }
  logout() {
    this.emit(PES`auth/logout:d`, this.user);
    this.api.token = null;
    this.user = null;
  }
  async register(u) {
    var _a;
    if (!u.username || !u.password) throw new Error("Cannot register user, missing username or password");
    const user = await this.api.request({ url: "/api/auth/register", body: { ...u } });
    if ((_a = user == null ? void 0 : user.image) == null ? void 0 : _a.startsWith("/")) user.image = `${this.api.url}${user.image}?token=${this.api.token}`;
    this.emit(PES`auth/register:c`, user);
    return user;
  }
  reset(emailOrPass, token) {
    if (!emailOrPass) throw new Error("Cannot reset password, missing email or token");
    return this.api.request({
      url: "/api/auth/reset",
      headers: { "Authorization": token ? `Bearer ${token}` : void 0 },
      body: {
        email: token ? void 0 : emailOrPass,
        password: token ? emailOrPass : void 0
      }
    }).then(() => {
      this.emit(PES`auth/reset:${token ? "u" : "c"}`, token || emailOrPass);
    });
  }
  async session(token, set = false) {
    if (!token) token = this.api.token;
    const session = await this.api.request({
      url: "/api/auth/session",
      headers: token ? { "Authorization": `Bearer ${token}` } : void 0
    });
    this.emit(PES`auth/session:r`, session);
    if (set) {
      this.api.token = token;
      if (session == null ? void 0 : session.user) session.user.image = `${this.api.url}${session.user.image}?token=${this.api.token}`;
      this.user = (session == null ? void 0 : session.user) || null;
      this.permissions = (session == null ? void 0 : session.permissions) || [];
      if (session) this.emit(PES`auth/login:c`, session.user);
    }
    return session;
  }
  async updatePassword(username, password, oldPassword) {
    if (!username || !password) throw new Error("Cannot update password, missing username or password");
    return this.api.request({
      url: "/api/auth/password",
      body: { username, password, oldPassword }
    }).then((resp) => {
      this.emit(PES`auth/reset:u`, resp == null ? void 0 : resp.token);
      if (resp == null ? void 0 : resp.token) this.api.token = resp.token;
    });
  }
  // TOTP ============================================================================================================
  disableTotp(username) {
    return this.api.request({ url: `/api/auth/totp/${username}`, method: "DELETE" });
  }
  resetTotp(username) {
    return this.api.request({ url: `/api/auth/totp/${username}`, method: "POST" });
  }
  setupTotp(username, method = "app", totp) {
    return this.api.request({ url: `/api/auth/totp/${username}`, body: clean({
      method,
      totp
    }) });
  }
}
class Client extends PathEventEmitter {
  constructor(api, settings) {
    super();
    __publicField(this, "_notifications", false);
    __publicField(this, "_platform");
    __publicField(this, "_pwa");
    this.api = api;
    this.settings = settings;
    this.pushSubscription.then((resp) => this.notifications = !!resp);
  }
  get mobile() {
    return ["android", "ios"].includes(this.platform);
  }
  get notifications() {
    return this._notifications;
  }
  set notifications(enabled) {
    this._notifications = enabled;
    this.emit(PES`client/notifications:${enabled ? "c" : "d"}`, enabled);
  }
  get platform() {
    if (!this._platform) {
      const userAgent = navigator.userAgent || navigator.vendor;
      if (/windows/i.test(userAgent)) this._platform = "windows";
      else if (/android/i.test(userAgent)) this._platform = "android";
      else if (/iPad|iPhone|iPod/.test(userAgent)) this._platform = "ios";
      else if (/macintosh|mac os x/i.test(userAgent)) this._platform = "mac";
      else if (/linux/i.test(userAgent)) this._platform = "linux";
      else this._platform = "unknown";
    }
    return this._platform;
  }
  get pushSubscription() {
    if (!navigator.serviceWorker.controller) return Promise.resolve(null);
    return navigator.serviceWorker.ready.then((sw) => sw.pushManager.getSubscription());
  }
  get pwa() {
    if (this._pwa == null)
      this._pwa = window.matchMedia("(display-mode: standalone)").matches || (navigator == null ? void 0 : navigator.standalone) || document.referrer.includes("android-app://");
    return this._pwa;
  }
  async inject(reload = false, firstLoad = true) {
    var _a, _b, _c, _d, _e, _f, _g, _h;
    let settings = this.settings.cache;
    if (firstLoad) this.settings.all(false, reload).then(() => this.inject(false, false)).catch(() => {
    });
    if (!document.querySelector('meta[name="mobile-web-app-capable"]')) {
      const meta = document.createElement("meta");
      meta.name = "mobile-web-app-capable";
      meta.content = "yes";
      document.head.append(meta);
    }
    if (!document.querySelector('meta[name="apple-mobile-web-app-status-bar-style"]')) {
      const meta = document.createElement("meta");
      meta.name = "apple-mobile-web-app-status-bar-style";
      meta.content = "default";
      document.head.append(meta);
    }
    if (!document.querySelector('meta[name="apple-mobile-web-app-title"]')) {
      const meta = document.createElement("meta");
      meta.name = "apple-mobile-web-app-title";
      meta.content = settings["title"];
      document.head.append(meta);
    }
    if (!document.querySelector('link[rel="apple-touch-icon"]')) {
      const meta = document.createElement("link");
      meta.rel = "apple-touch-icon";
      meta.href = settings["logo"];
      document.head.append(meta);
    }
    if (!document.querySelector('link[rel="apple-touch-startup-image"]')) {
      const meta = document.createElement("link");
      meta.rel = "apple-touch-startup-image";
      meta.href = settings["logo"];
      document.head.append(meta);
    }
    window.document.body.classList.add(((_a = settings["theme"]) == null ? void 0 : _a.darkMode) ? "theme-dark" : "theme-light");
    window.document.body.classList.remove(((_b = settings["theme"]) == null ? void 0 : _b.darkMode) ? "theme-light" : "theme-dark");
    if (settings["title"])
      window.document.querySelectorAll(".momentum-title").forEach((el) => el.innerText = settings["title"]);
    if (settings["description"])
      window.document.querySelectorAll(".momentum-description").forEach((el) => el.innerText = settings["description"]);
    if (settings.version)
      window.document.querySelectorAll(".momentum-version").forEach((el) => el.innerText = settings["version"]);
    if (settings["logo"]) {
      window.document.querySelectorAll(".momentum-logo").forEach((el) => {
        el.src = settings["logo"];
        el.href = settings["logo"];
      });
    }
    if (settings["theme"]) {
      let style = window.document.querySelector("#momentum-theme");
      if (!style) {
        style = window.document.createElement("style");
        style.id = "momentum-theme";
        window.document.head.append(style);
      }
      style.innerHTML = `
			:root {
				--theme-bg-primary: ${(_c = settings["theme"]) == null ? void 0 : _c.background};
				--theme-bg-secondary: ${((_d = settings["theme"]) == null ? void 0 : _d.darkMode) ? "#2e2e2e" : "#ffffff"};
				--theme-primary: ${(_e = settings["theme"]) == null ? void 0 : _e.primary};
				--theme-secondary: ${(_f = settings["theme"]) == null ? void 0 : _f.secondary};
				--theme-text: ${((_g = settings["theme"]) == null ? void 0 : _g.darkMode) ? "#ffffff" : "#000000"};
				--theme-muted: ${((_h = settings["theme"]) == null ? void 0 : _h.darkMode) ? "#cccccc" : "#6c757d"};
			}`;
    }
    if (settings["pwa"]) {
      const link = window.document.createElement("link");
      link.setAttribute("rel", "manifest");
      link.setAttribute("href", this.settings.api.url + "/manifest.json");
      window.document.head.append(link);
      setTimeout(() => {
        const dismissed = !!localStorage.getItem("momentum:install-prompt");
        if (!dismissed && !this.pwa && this.mobile) this.pwaPrompt();
      }, 500);
    }
    this.emit(PES`client/inject:c`, this.platform);
  }
  pwaPrompt(platform) {
    const url = this.settings.api.url;
    const settings = this.settings.cache;
    const android = (platform || this.platform) == "android";
    let style = document.querySelector("style.momentum-pwa");
    if (!style) {
      style = document.createElement("style");
      style.innerHTML = `
		.momentum-pwa-prompt-backdrop {
			position: fixed;
			display: relative;
			left: 0;
			top: 0;
			right: 0;
			bottom: 0;
			background: rgba(0,0,0,.5);
			z-index: 9999;
			animation: fadeIn 0.5s ease-in-out forwards;
			opacity: 0;
		}
		.momentum-pwa-prompt-backdrop.exit {
			animation: fadeOut 0.5s ease-in-out forwards !important;
		}
		.momentum-pwa-prompt {
			position: fixed;
			background: #fff;
			color: black;
			bottom: 0;
			left: 50%;
			width: min(100vw, 450px);
			transform: translate(-50%, 0);
			animation: slideUp 0.5s ease-in-out forwards;
		}
		.momentum-pwa-prompt.exit {
			animation: slideDown 0.5s ease-in-out forwards !important;
		}
		.momentum-pwa-prompt img {
			width: 18px;
			height: 18px;
		}
		.momentum-pwa-prompt h1 {
			font-size: 1.25rem;
			font-weight: bold;
		}
		.momentum-pwa-prompt-close {
			position: absolute;
			right: 5px;
			top: 10px;
			background: transparent;
			border: none;
			cursor: pointer;
		}

		@keyframes fadeIn {
		  from { opacity: 0; }
		  to { opacity: 1; }
		}
		@keyframes fadeOut {
		  from { opacity: 1; }
		  to { opacity: 0; }
		}

		@keyframes slideUp {
		  from { transform: translate(-50%, 100%); }
		  to { transform: translate(-50%, 0); }
		}
		@keyframes slideDown {
		  from { transform: translate(-50%, 0); }
		  to { transform: translate(-50%, 100%); }
		}
		`;
      document.head.append(style);
    }
    const backdrop = document.createElement("div");
    backdrop.classList.add("momentum-pwa-prompt-backdrop");
    const prompt = document.createElement("div");
    prompt.classList.add("momentum-pwa-prompt");
    prompt.innerHTML = `
			<div style="display: flex; padding: 1rem; align-items: center">
			  <img src="${url}${settings.logo}" alt="Logo" style="height: 30px; width: auto; margin-right: .5rem;" />
			  <h1 style="margin: 0">Install ${settings.title}</h1>
			</div>
			<div style="display: flex; flex-direction: column; align-items: center">
			  <div style="border-top: 2px solid #00000020; border-bottom: 2px solid #00000020; padding: 0 1rem">
				<p style="margin-top: 1rem; text-align: center">This website can be installed as an App! Add it to your home screen for quick access & fullscreen use.</p>
			  </div>
			  <table style="margin: 1.5rem 0">
				<tr>
				  <td style="width: 50px; text-align: center">
				    ${android ? '<svg xmlns="http://www.w3.org/2000/svg" height="24px" viewBox="0 -960 960 960" width="24px" fill="black" style="transform: scale(1.5)"><path d="M480-160q-33 0-56.5-23.5T400-240q0-33 23.5-56.5T480-320q33 0 56.5 23.5T560-240q0 33-23.5 56.5T480-160Zm0-240q-33 0-56.5-23.5T400-480q0-33 23.5-56.5T480-560q33 0 56.5 23.5T560-480q0 33-23.5 56.5T480-400Zm0-240q-33 0-56.5-23.5T400-720q0-33 23.5-56.5T480-800q33 0 56.5 23.5T560-720q0 33-23.5 56.5T480-640Z"/></svg>' : '<svg viewBox="0 0 566 670" xmlns="http://www.w3.org/2000/svg" fill="#0B76FC" height="40px"><path d="M255 12c4-4 10-8 16-8s12 3 16 8l94 89c3 4 6 7 8 12 2 6 0 14-5 19-7 8-20 9-28 2l-7-7-57-60 2 54v276c0 12-10 22-22 22-12 1-24-10-23-22V110l1-43-60 65c-5 5-13 8-21 6a19 19 0 0 1-16-17c-1-7 2-13 7-18l95-91z" /><path d="M43 207c16-17 40-23 63-23h83v46h-79c-12 0-25 3-33 13-8 9-10 21-10 33v260c0 13 0 27 6 38 5 12 18 18 30 19l14 1h302c14 0 28 0 40-8 11-7 16-21 16-34V276c0-11-2-24-9-33-8-10-22-13-34-13h-78v-46h75c13 0 25 1 37 4 16 4 31 13 41 27 11 17 14 37 14 57v280c0 20-3 41-15 58a71 71 0 0 1-45 27c-11 2-23 3-34 3H109c-19-1-40-4-56-15-14-9-23-23-27-38-4-12-5-25-5-38V270c1-22 6-47 22-63z" /></svg>'}
				  </td>
				  <td>
					<p style="margin: 1rem 0">1) ${android ? "Open the dropdown menu" : 'Press the "Share" button'}</p>
				  </td>
				</tr>
				<tr>
				  <td style="width: 50px; text-align: center">
				    ${android ? '<svg xmlns="http://www.w3.org/2000/svg" height="24px" viewBox="0 -960 960 960" width="24px" fill="black" style="transform: scale(-1.5, 1.5)"><path d="M280-40q-33 0-56.5-23.5T200-120v-720q0-33 23.5-56.5T280-920h280v80H280v40h280v80H280v480h400v-80h80v200q0 33-23.5 56.5T680-40H280Zm0-120v40h400v-40H280Zm440-240L520-600l56-56 104 104v-288h80v288l104-104 56 56-200 200ZM280-800v-40 40Zm0 640v40-40Z"/></svg>' : '<svg viewBox="0 0 578 584" xmlns="http://www.w3.org/2000/svg" fill="black" height="34px"><path d="M101 35l19-1h333c12 0 23 0 35 3 17 3 34 12 44 27 13 16 16 38 16 58v329c0 19 0 39-8 57a65 65 0 0 1-37 37c-18 7-38 7-57 7H130c-21 1-44 0-63-10-14-7-25-20-30-34-6-15-8-30-8-45V121c1-21 5-44 19-61 13-16 33-23 53-25m7 46c-10 1-19 6-24 14-7 8-9 20-9 31v334c0 12 2 25 10 34 9 10 23 12 35 12h336c14 1 30-3 38-15 6-9 8-20 8-31V125c0-12-2-24-10-33-9-9-22-12-35-12H121l-13 1z" /><path d="M271 161c9-11 31-10 38 4 3 5 3 11 3 17v87h88c7 0 16 1 21 7 6 6 7 14 6 22a21 21 0 0 1-10 14c-5 4-11 5-17 5h-88v82c0 7-1 15-6 20-10 10-29 10-37-2-3-6-4-13-4-19v-81h-87c-8-1-17-3-23-9-5-6-6-15-4-22a21 21 0 0 1 11-14c6-3 13-3 19-3h84v-88c0-7 1-14 6-20z" /></svg>'}
				  </td>
				  <td>
					<p style="margin: 1rem 0">2) Press "Add to Home Screen"</p>
				  </td>
				</tr>
			  </table>
			</div>`;
    const close = document.createElement("button");
    close.classList.add("momentum-pwa-prompt-close");
    close.innerText = "X";
    close.onclick = () => {
      prompt.classList.add("exit");
      backdrop.classList.add("exit");
      localStorage.setItem("momentum:install-prompt", "dismissed");
      setTimeout(() => {
        prompt.remove();
        backdrop.remove();
        this.emit(PES`client/pwa:d`, platform);
      }, 500);
    };
    prompt.append(close);
    backdrop.append(prompt);
    document.body.append(backdrop);
    this.emit(PES`client/pwa:c`, platform);
  }
  async enableNotifications() {
    const granted = await Notification.requestPermission();
    if (!granted) return null;
    const sw = await navigator.serviceWorker.ready;
    const subscription = (await sw.pushManager.subscribe({
      userVisibleOnly: true,
      applicationServerKey: this.settings.cache["push_public_key"]
    })).toJSON();
    return this.api.request({ url: "/api/notifications", body: subscription }).then(() => this.notifications = true);
  }
  async disableNotifications() {
    var _a;
    const subscription = await this.pushSubscription;
    subscription == null ? void 0 : subscription.unsubscribe();
    return this.api.request({ url: "/api/notifications", method: "DELETE", body: {
      p256dh: (_a = subscription == null ? void 0 : subscription.toJSON().keys) == null ? void 0 : _a["p256dh"]
    } }).then(() => this.notifications = false);
  }
}
class Data extends PathEventEmitter {
  constructor(api) {
    super();
    __publicField(this, "api");
    this.api = typeof api == "string" ? new Api(api) : api;
  }
  create(collection, document2) {
    if (!collection || !document2) throw new Error("Cannot create document, missing collection or document");
    return this.api.request({
      url: `/api/` + PES`data/${collection}`,
      method: "POST",
      body: document2
    }).then((resp) => {
      this.emit(PES`data/${collection}:c`, resp);
      return resp;
    });
  }
  delete(collection, id) {
    if (!collection || !id) throw new Error("Cannot delete document, missing collection or ID");
    return this.api.request({
      url: `/api/` + PES`data/${collection}/${id}`,
      method: "DELETE"
    }).then(() => this.emit(PES`data/${collection}/${id}:d`, id));
  }
  read(collection, id) {
    if (!collection) throw new Error("Cannot read documents, missing collection");
    return this.api.request({ url: `/api/` + PES`data/${collection}/${id}` }).then((resp) => {
      this.emit(PES`data/${collection}/${id}:r`, collection, resp);
      return resp;
    });
  }
  update(collection, document2, append) {
    if (!collection || !document2) throw new Error("Cannot update document, missing collection or document");
    return this.api.request({
      url: `/api/` + PES`data/${collection}/${document2._id}`,
      method: append ? "PATCH" : "PUT",
      body: document2
    }).then((resp) => {
      this.emit(PES`data/${collection}/${document2._id}:u`, resp);
      return resp;
    });
  }
  raw(collection, query) {
    if (!collection || !query) throw new Error("Cannot execute raw query, missing collection or query");
    const mode = query.operand.startsWith("find") ? "r" : query.operand == "insert" ? "c" : query.operand.startsWith("delete") ? "d" : "u";
    return this.api.request({ url: `/api/` + PES`data/${collection}` + "?raw", body: query }).then((resp) => {
      this.emit(PES`data/${collection}:${mode}`, resp);
      return resp;
    });
  }
  // Schema ==========================================================================================================
  deleteSchema(path) {
    if (!path) throw new Error("Cannot delete schema, missing collection path");
    return this.api.request({ url: `/api/` + PES`schema/${path}`, method: "DELETE" }).then(() => this.emit(PES`schema/${path}:d`, path));
  }
  readSchema(pathOrTree) {
    return this.api.request({ url: "/api/" + PES`schema/${typeof pathOrTree == "string" ? pathOrTree : ""}` + (pathOrTree === true ? `?tree=${pathOrTree}` : "") }).then((resp) => {
      this.emit(PES`schema/${typeof pathOrTree == "string" ? pathOrTree : ""}:r`, resp);
      return resp;
    });
  }
  updateSchema(schema) {
    if (!schema.path) throw new Error("Cannot update schema, missing collection path");
    return this.api.request({ url: "/api/" + PES`schema/${schema.path}`, body: schema }).then((resp) => {
      this.emit(PES`schema/${schema.path}:${schema._id ? "u" : "c"}`, resp);
      return resp;
    });
  }
}
class Email extends PathEventEmitter {
  constructor(api) {
    super();
    __publicField(this, "api");
    this.api = typeof api == "string" ? new Api(api) : api;
  }
  send(email) {
    var _a;
    if (!email.to && !email.bcc || !email.body) throw new Error("Cannot send email, missing address or body");
    return this.api.request({ url: "/api/" + PES`email/${(_a = email.body) == null ? void 0 : _a.template}`, body: email }).then((response) => {
      var _a2;
      this.emit(PES`email/${(_a2 = email.body) == null ? void 0 : _a2.template}:c`, { email, response });
      return response;
    });
  }
}
class Forms extends PathEventEmitter {
  constructor(api) {
    super();
    __publicField(this, "api");
    __publicField(this, "cache", new Cache("_id"));
    this.api = typeof api == "string" ? new Api(api) : api;
  }
  all(pathOrTree) {
    return this.api.request({ url: "/api/" + PES`forms/${typeof pathOrTree == "string" ? pathOrTree : ""}` + (pathOrTree === true ? `?tree=${pathOrTree}` : "") }).then((resp) => {
      this.emit(PES`forms/${typeof pathOrTree == "string" ? pathOrTree : ""}:r`, resp);
      return resp;
    });
  }
  delete(key) {
    if (!key) throw new Error("Cannot delete form, missing path");
    return this.api.request({ url: `/api/` + PES`forms/${key}`, method: "DELETE" }).then(() => {
      this.cache.delete(key);
      this.emit(PES`forms/${key}:d`, key);
    });
  }
  read(path, reload = false) {
    if (!path) throw new Error("Cannot read form, missing path");
    if (!reload && this.cache.get(path)) return Promise.resolve(this.cache.get(path));
    return this.api.request({ url: `/api/` + PES`forms/${path}` }).then((form) => {
      if (form) this.cache.add(form);
      this.emit(PES`forms/${path}:r`, form);
      return form;
    });
  }
  update(form) {
    if (!form.path) throw new Error("Cannot update form, missing path");
    return this.api.request({ url: `/api/` + PES`forms/${form.path}`, body: form }).then((f) => {
      if (f) this.cache.add(f);
      this.emit(`/api/` + PES`settings/${form.path}:${form._id ? "u" : "c"}`, f);
      return f;
    });
  }
}
class Groups extends PathEventEmitter {
  constructor(api) {
    super();
    __publicField(this, "api");
    __publicField(this, "cache", new Cache("name"));
    this.api = typeof api == "string" ? new Api(api) : api;
  }
  async all(reload) {
    if (!reload && this.cache.complete) return this.cache.all();
    return this.api.request({ url: `/api/` + PES`groups` }).then((resp) => {
      this.cache.addAll(resp);
      this.emit(PES`groups:r`, resp || []);
      return resp;
    });
  }
  delete(name) {
    if (!name) throw new Error("Cannot delete group, missing name");
    return this.api.request({
      url: `/api/` + PES`groups/${name}`,
      method: "DELETE"
    }).then(() => {
      this.cache.delete(name);
      this.emit(PES`groups/${name}:d`);
    });
  }
  create(group) {
    if (!group.name) throw new Error("Cannot create group, missing name");
    return this.api.request({
      url: `/api/` + PES`groups/${group.name}`,
      method: "POST",
      body: group
    }).then((resp) => {
      this.cache.add(resp);
      this.emit(PES`groups/${group.name}:c`, resp);
      return resp;
    });
  }
  async read(name, reload) {
    if (!name) throw new Error("Cannot read group, missing name");
    if (!reload && this.cache.get(name)) return this.cache.get(name);
    return this.api.request({ url: `/api/` + PES`groups/${name}` }).then((resp) => {
      this.cache.add(resp);
      this.emit(PES`groups/${name}:r`, resp);
      return resp;
    });
  }
  update(group) {
    if (!group.name) throw new Error("Cannot update group, missing name");
    return this.api.request({
      url: `/api/` + PES`groups/${group.name}`,
      method: "PATCH",
      body: group
    }).then((resp) => {
      this.cache.add(resp);
      this.emit(PES`groups/${group.name}:u`, resp);
      return resp;
    });
  }
}
class Logger extends PathEventEmitter {
  constructor(api, channel, logLevel) {
    super();
    __publicField(this, "api");
    __publicField(this, "console", {
      debug: console.debug,
      log: console.log,
      info: console.info,
      warn: console.warn,
      error: console.error
    });
    this.channel = channel;
    if (channel.toLowerCase() == "server") throw new Error('"Server" namespace is reserved');
    this.api = typeof api == "string" ? new Api(api) : api;
    if (logLevel != null && logLevel && logLevel != "NONE") {
      if (LOG_LEVEL[logLevel] >= 0) {
        console.error = (...args) => {
          this.console.error(...args);
          this.error(...args);
        };
        window.addEventListener("unhandledrejection", async (event) => {
          var _a, _b, _c, _d;
          let log = ((_a = event.reason) == null ? void 0 : _a.stack) || event.reason;
          if ((_b = event.reason) == null ? void 0 : _b.url)
            log = `${event.reason.method} ${event.reason.url} -> ${event.reason.code}

${log}`;
          ((_c = event.reason) == null ? void 0 : _c.code) == null || ((_d = event.reason) == null ? void 0 : _d.code) >= 500 ? this.error(log) : this.warn(log);
        });
      }
      if (LOG_LEVEL[logLevel] >= 1) {
        console.warn = (...args) => {
          this.console.warn(...args);
          this.warn(...args);
        };
      }
      if (LOG_LEVEL[logLevel] >= 2) {
        console.info = (...args) => {
          this.console.info(...args);
          this.info(...args);
        };
      }
      if (LOG_LEVEL[logLevel] >= 3) {
        console.log = (...args) => {
          this.console.log(...args);
          this.log(...args);
        };
      }
      if (LOG_LEVEL[logLevel] >= 4) {
        console.debug = (...args) => {
          this.console.debug(...args);
          this.debug(...args);
        };
      }
    }
  }
  buildLog(level, log) {
    return {
      time: (/* @__PURE__ */ new Date()).getTime(),
      level,
      log,
      ctx: {
        cores: navigator.hardwareConcurrency,
        mem: navigator == null ? void 0 : navigator.deviceMemory,
        res: [window.innerWidth, window.innerHeight],
        url: location.href
      }
    };
  }
  create(log, channel = this.channel) {
    if (channel.toLowerCase() == "server") throw new Error('"Server" namespace is reserved');
    return this.api.request({ url: `/api/` + PES`logs/${channel}`, body: log }).then(() => this.emit(PES`logs/${channel}:c`, log)).catch(() => {
    });
  }
  channels() {
    return this.api.request({ url: "/api/" + PES`logs/channels` });
  }
  delete(channel = this.channel) {
    return this.api.request({ url: `/api/` + PES`logs/${channel}`, method: "DELETE" }).then(() => this.emit(PES`logs/${channel}:d`));
  }
  read(channel = this.channel) {
    return this.api.request({ url: `/api/` + PES`logs/${channel}` }).then((logs) => {
      this.emit(PES`logs/${channel}:r`, logs);
      return logs;
    });
  }
  // Console =========================================================================================================
  debug(log, channel = this.channel) {
    return this.create(this.buildLog(LOG_LEVEL.DEBUG, log), channel);
  }
  log(log, channel = this.channel) {
    return this.create(this.buildLog(LOG_LEVEL.LOG, log), channel);
  }
  info(log, channel = this.channel) {
    return this.create(this.buildLog(LOG_LEVEL.INFO, log), channel);
  }
  warn(log, channel = this.channel) {
    return this.create(this.buildLog(LOG_LEVEL.WARN, log), channel);
  }
  error(log, channel = this.channel) {
    return this.create(this.buildLog(LOG_LEVEL.ERROR, log), channel);
  }
}
class Payments extends PathEventEmitter {
  constructor(api, secret) {
    super();
    __publicField(this, "api");
    __publicField(this, "stripe");
    this.api = typeof api == "string" ? new Api(api) : api;
    const setup = (retry = 1) => {
      try {
        if (!Stripe) throw new Error("Stripe not added");
        this.stripe = Stripe(secret);
      } catch (err) {
        if (retry > 0) setTimeout(() => setup(retry--), 250);
        else console.warn("Stripe failed, did you add the library & setup your API key?", err);
      }
    };
    setup();
  }
  async create(amount, custom = {}) {
    if (!amount) throw new Error("Please specify a valid amount`");
    const request = await this.api.request({ url: "/api/payments", body: {
      amount,
      custom
    } });
    this.emit(PES`payments:c`, request.data.clientSecret);
    return request.data.clientSecret;
  }
  async createForm(element, amount, custom) {
    const token = await this.create(amount, custom);
    const form = this.stripe.elements({ clientSecret: token });
    form.create("payment").mount(element);
    return () => this.stripe.confirmPayment({
      elements: form,
      redirect: "if_required",
      confirmParams: { return_url: location.origin }
    });
  }
  async history(username) {
    const history = await this.api.request({ url: `/api/` + PES`payments/${username}` });
    this.emit(PES`payments/${username}:r`, history);
    return history;
  }
}
class Pdf extends PathEventEmitter {
  constructor(api) {
    super();
    __publicField(this, "api");
    this.api = typeof api == "string" ? new Api(api) : api;
  }
  createPdf(body, options) {
    return this.api.request({ url: `/api/` + PES`pdf`, body: { ...body, options }, decode: false }).then(async (resp) => {
      const blob = await resp.blob();
      if (options == null ? void 0 : options.download) {
        let filename = (options == null ? void 0 : options.filename) || timestampFilename();
        if (!filename.endsWith(".pdf")) filename += ".pdf";
        const url = URL.createObjectURL(blob);
        downloadUrl(url, filename);
        URL.revokeObjectURL(url);
      }
      this.emit(PES`pdf:c`, blob);
      return blob;
    });
  }
  fromHtml(html, options = {}) {
    if (!html) throw new Error("Cannot create PDF, missing HTML");
    return this.createPdf({ html }, options);
  }
  fromTemplate(template, data, options = {}) {
    if (!template) throw new Error("Cannot create PDF, missing template");
    return this.createPdf({ template, data }, options);
  }
  fromUrl(url, options = {}) {
    if (!url) throw new Error("Cannot create PDF, missing URL");
    return this.createPdf({ url }, options);
  }
}
const _Socket = class _Socket {
  constructor(api) {
    __publicField(this, "api");
    __publicField(this, "url");
    __publicField(this, "connection");
    __publicField(this, "open", false);
    this.api = typeof api == "string" ? new Api(api) : api;
    this.url = this.api.url.replace("http", "ws");
    this.api.on("api/token", () => this.connect());
    this.connect();
  }
  close() {
    var _a;
    if (this.open) console.debug("Disconnected from Momentum");
    this.open = false;
    (_a = this.connection) == null ? void 0 : _a.close();
    this.connection = void 0;
  }
  connect() {
    if (this.open) this.close();
    if (navigator.onLine) {
      this.connection = new WebSocket(this.url + (this.api.token ? `?token=${this.api.token}` : ""));
      this.connection.onclose = this.close;
      this.connection.onmessage = this.handle;
      this.connection.onopen = () => {
        this.open = true;
        clearTimeout(timeout);
        console.debug("Connected to Momentum");
      };
    }
    const timeout = setTimeout(() => {
      if (this.open) return;
      this.close();
      this.connect();
    }, _Socket.pollingSpeed);
  }
  handle(...args) {
    console.log(args);
  }
  send(channel, payload) {
    var _a;
    (_a = this.connection) == null ? void 0 : _a.send(JSON.stringify({
      token: this.api.token,
      channel,
      payload
    }));
  }
};
__publicField(_Socket, "pollingSpeed", 3e4);
let Socket = _Socket;
let Storage$1 = class Storage2 extends PathEventEmitter {
  constructor(api) {
    super();
    __publicField(this, "api");
    this.api = typeof api == "string" ? new Api(api) : api;
  }
  copy(source, destination) {
    if (!source || !destination) throw new Error("Cannot copy file or folder, missing source or destination");
    return this.api.request({ url: "/api/" + PES`storage/${destination}`, body: { from: source } }).then((response) => {
      this.emit(PES`storage/${destination}:c`, response);
      return response;
    });
  }
  delete(path) {
    if (!path) throw new Error("Cannot delete file or folder, missing path");
    return this.api.request({ url: "/api/" + PES`storage/${path}`, method: "DELETE" }).then(() => {
      this.emit(PES`storage/${path}:d`, path);
    });
  }
  download(path, opts = {}) {
    if (!path) throw new Error("Cannot download file, missing path");
    return this.api.request({ ...opts, url: "/api/" + PES`storage/${path}`, decode: false }).then(async (response) => {
      const blob = await response.blob();
      const name = opts.downloadAs || path.split("/").pop();
      this.emit(PES`storage/${path}:r`, blob);
      downloadFile(blob, name);
      return response;
    });
  }
  list(path) {
    if (!path) path = "/";
    return this.api.request({ url: "/api/" + PES`storage/${path}` + "?list" }).then((resp) => {
      this.emit(PES`storage/${path}:r`, resp);
      return resp;
    });
  }
  open(path, target = "_blank") {
    if (!path) throw new Error("Cannot download file, missing path");
    const link = `${this.api.url}/api/` + PES`storage/${path}` + (this.api.token ? `?token=${this.api.token}` : "");
    if (!target) return link;
    this.emit(PES`storage/${path}:r`, path);
    return window.open(link, target);
  }
  mkdir(path) {
    if (!path) throw new Error("Cannot make directory, missing path");
    return this.api.request({ url: "/api/" + PES`storage/${path}`, body: { directory: true } }).then((resp) => {
      this.emit(PES`storage/${path}:c`, resp);
      return resp;
    });
  }
  move(source, destination) {
    if (!source || !destination) throw new Error("Cannot move file or folder, missing source or destination");
    if (source == destination) return this.list(destination);
    return this.api.request({ url: "/api/" + PES`storage/${source}`, method: "PATCH", body: { move: destination } }).then((response) => {
      this.emit(PES`storage/${source}:u`, response);
      return response;
    });
  }
  upload(files, opts) {
    return new PromiseProgress(async (res, rej, prog) => {
      if (!files) files = await fileBrowser(typeof opts == "object" ? opts : void 0);
      if (!files || Array.isArray(files) && !files.length) return [];
      const path = (opts && typeof opts == "object" ? opts == null ? void 0 : opts.path : opts) || "/";
      return uploadWithProgress({
        url: `${this.api.url}/api/` + PES`storage/${path}`,
        files: makeArray(files),
        headers: this.api.headers
      }).onProgress((p) => {
        prog(p);
      }).then((resp) => {
        this.emit(PES`storage/${path}:c`, resp);
        res(resp);
      }).catch((err) => rej(err));
    });
  }
};
class Users extends PathEventEmitter {
  constructor(api) {
    super();
    __publicField(this, "api");
    __publicField(this, "cache", new Cache("username"));
    this.api = typeof api == "string" ? new Api(api) : api;
  }
  async all(reload) {
    if (!reload && this.cache.complete) return this.cache.all();
    return this.api.request({ url: "/api/" + PES`users` }).then((resp) => {
      resp == null ? void 0 : resp.forEach((r) => {
        r.image = this.api.url + r.image + `?token=${this.api.token}`;
        return r;
      });
      this.cache.addAll(resp);
      this.emit(PES`users:r`, resp || []);
      return resp;
    });
  }
  delete(username) {
    if (!username) throw new Error("Cannot delete user, missing username");
    return this.api.request({
      url: "/api/" + PES`users/${username}`,
      method: "DELETE"
    }).then(() => {
      this.cache.delete(username);
      this.emit(PES`users/${username}:d`, username);
    });
  }
  async read(username, reload) {
    if (!username) throw new Error("Cannot read user, missing username");
    if (!reload && this.cache.get(username)) return this.cache.get(username);
    return this.api.request({ url: "/api/" + PES`users/${username}` }).then((resp) => {
      if (resp) {
        resp.image = this.api.url + resp.image + `?token=${this.api.token}`;
        this.cache.add(resp);
      }
      this.emit(PES`users/${username}:r`, resp);
      return resp;
    });
  }
  update(user) {
    if (!user.username) throw new Error("Cannot update user, missing username");
    return this.api.request({
      url: `/api/` + PES`users/${user.username}`,
      method: "PATCH",
      body: user
    }).then((resp) => {
      if (resp) {
        resp.image = this.api.url + resp.image + `?token=${this.api.token}`;
        this.cache.add(resp);
      }
      this.emit(PES`users/${user.username}:${resp._id ? "u" : "c"}`, resp);
      return resp;
    });
  }
  uploadImage(username, file) {
    if (!username || !file) throw new Error("Cannot update user image, missing username or file");
    return uploadWithProgress({
      url: this.api.url + `/api/` + PES`users/${username}/image`,
      files: [file],
      headers: this.api.headers
    });
  }
}
class Settings extends PathEventEmitter {
  constructor(api) {
    super();
    __publicField(this, "api");
    __publicField(this, "cache");
    this.api = typeof api == "string" ? new Api(api) : api;
    this.cache = new Cache("key", { storageKey: `momentum:settings:${this.api.host}` });
    this.api.on("api/token", () => this.all(false, true));
  }
  async all(detailed, reload) {
    if (!reload && !detailed && this.cache.complete) return this.cache;
    return this.api.request({ url: `/api/` + PES`settings` + (detailed ? "?detailed" : "") }).then((resp) => {
      this.cache.clear();
      if (resp) Object.keys(resp).forEach((key) => this.cache.set(key, detailed ? resp[key].value : resp[key]));
      this.cache.complete = true;
      this.emit(PES`settings:r`, resp || []);
      return resp;
    });
  }
  delete(key) {
    if (!key) throw new Error("Cannot delete setting, missing key");
    return this.api.request({ url: `/api/` + PES`settings/${key}`, method: "DELETE" }).then(() => {
      this.cache.delete(key);
      this.emit(PES`settings/${key}:d`, key);
    });
  }
  read(key, reload = false) {
    if (!key) throw new Error("Cannot read setting, missing key");
    if (!reload && this.cache.get(key)) return this.cache.get(key);
    return this.api.request({ url: `/api/` + PES`settings/${key}` }).then((variable) => {
      if (variable) this.cache.set(variable.key, variable.value);
      this.emit(PES`settings/${key}:r`, variable);
      return variable;
    });
  }
  update(variable) {
    if (!variable.key) throw new Error("Cannot update setting, missing key");
    return this.api.request({ url: `/api/` + PES`settings/${variable.key}`, body: variable }).then((variable2) => {
      if (variable2) this.cache.set(variable2.key, variable2.value);
      this.emit(`/api/` + PES`settings/${variable2.key}:${variable2._id ? "u" : "c"}`, variable2);
      return variable2;
    });
  }
}
class Static extends PathEventEmitter {
  constructor(api) {
    super();
    __publicField(this, "api");
    this.api = typeof api == "string" ? new Api(api) : api;
  }
  delete(path) {
    if (!path) throw new Error("Cannot delete static asset, missing path");
    return this.api.request({ url: `/api/` + PES`static/${path}`, method: "DELETE" }).then(() => {
      this.emit(PES`static/${path}:d`, path);
    });
  }
  upload(files, path = "/") {
    if (!files) throw new Error("Cannot upload static assets, missing file");
    return new PromiseProgress(async (res, rej, prog) => {
      return uploadWithProgress({
        url: this.api.url + "/api/" + PES`static/${path}`,
        files: makeArray(files),
        headers: this.api.headers
      }).onProgress((p) => {
        prog(p);
      }).then((resp) => {
        this.emit(PES`static/${path}:c`, resp);
        res(resp);
      }).catch((err) => rej(err));
    });
  }
}
class Momentum extends PathEventEmitter {
  constructor(url, opts) {
    super();
    __publicField(this, "api");
    __publicField(this, "actions");
    __publicField(this, "ai");
    __publicField(this, "analytics");
    __publicField(this, "auth");
    __publicField(this, "client");
    __publicField(this, "data");
    __publicField(this, "email");
    __publicField(this, "forms");
    __publicField(this, "groups");
    __publicField(this, "logger");
    __publicField(this, "payments");
    __publicField(this, "pdf");
    __publicField(this, "settings");
    __publicField(this, "socket");
    __publicField(this, "static");
    __publicField(this, "storage");
    __publicField(this, "users");
    this.api = new Api(url, opts);
    this.actions = new Actions(this.api);
    this.ai = new Ai(this.api);
    this.analytics = new Analytics(this.api);
    this.auth = new Auth(this.api, { loginUrl: opts == null ? void 0 : opts.loginUrl });
    this.data = new Data(this.api);
    this.email = new Email(this.api);
    this.forms = new Forms(this.api);
    this.groups = new Groups(this.api);
    this.logger = new Logger(this.api, (opts == null ? void 0 : opts.app) || "client", opts == null ? void 0 : opts.logLevel);
    if (opts == null ? void 0 : opts.stripeSecret) this.payments = new Payments(this.api, opts.stripeSecret);
    this.pdf = new Pdf(this.api);
    this.settings = new Settings(this.api);
    if (opts == null ? void 0 : opts.socket) this.socket = new Socket(this.api);
    this.static = new Static(this.api);
    this.storage = new Storage$1(this.api);
    this.client = new Client(this.api, this.settings);
    this.users = new Users(this.api);
    this.relayEvents(this.actions);
    this.relayEvents(this.ai);
    this.relayEvents(this.analytics);
    this.relayEvents(this.api);
    this.relayEvents(this.auth);
    this.relayEvents(this.client);
    this.relayEvents(this.data);
    this.relayEvents(this.email);
    this.relayEvents(this.groups);
    this.relayEvents(this.logger);
    if (this.payments) this.relayEvents(this.payments);
    this.relayEvents(this.pdf);
    this.relayEvents(this.settings);
    this.relayEvents(this.static);
    this.relayEvents(this.storage);
    this.relayEvents(this.users);
    this.users.on(`*`, () => {
      if (!this.auth.user) return;
      const cached = this.users.cache.get(this.auth.user.username);
      if (cached) this.auth.user = cached;
    });
    if (!(opts == null ? void 0 : opts.disableWorker) && "serviceWorker" in navigator) {
      navigator.serviceWorker.register("/momentum-worker.js");
    }
  }
}
export {
  ActionType,
  Actions,
  Ai,
  Analytics,
  Api,
  Auth,
  Client,
  Data,
  Email,
  Forms,
  Groups,
  Logger,
  Momentum,
  PE,
  PES,
  PathEvent,
  PathEventEmitter,
  Payments,
  Pdf,
  Settings,
  Socket,
  Static,
  Storage$1 as Storage,
  Users
};
